//	AksPro - ObjectARX module for AutoCAD 2005
//	Copyright (C) 2005 Wiebe Leonid, All Rights Reserved.
//
//	This software is provided 'as-is', without any express or implied
//	warranty.  In no event will the authors be held liable for any
//	damages arising from the use of this software.
//
//	Permission is granted to anyone to use this software for any purpose,
//	including commercial applications, and to alter it and redistribute it
//	freely, subject to the following restrictions:
//
//	1.	The origin of this software must not be misrepresented; you must
//		not claim that you wrote the original software. If you use this
//		software in a product, an acknowledgment in the product
//		documentation would be appreciated but is not required.
//	2.	Altered source versions must be plainly marked as such, and must
//		not be misrepresented as being the original software.
//	3.	This notice may not be removed or altered from any source
//		distribution.

//	AksPro -  ObjectARX  AutoCAD 2005
//	  2005,  
//
//	     ; 
//	   /     
//	     GNU   , 
//	      ; 
//	  2   (  )    .
//
//	   ,    , 
//	        ; 
//	    ,  
//	       . 
//	      GNU.
//
//	        GNU 
//	   ;    ,      
//	(Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).



#include "stdafx.h"
#include "Dlg.h"

void processDatabase(int);
void processEntities(ads_name ss, long length, int iType);

extern char* vers;

extern PREFS prefCurSet;
extern PREFS prefFileSet;

extern bool bDragMode;


// Dlg dialog

IMPLEMENT_DYNAMIC(Dlg, CDialog)

Dlg::Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(Dlg::IDD, pParent)
	, m_R(prefFileSet.iAksType)
	, m_SegQty(prefFileSet.iSegQty)
	, m_SelElems(prefFileSet.iSelElems)
	, m_Mirror(prefFileSet.iMirror)
{

}

Dlg::~Dlg()
{
}

BOOL Dlg::Create( CWnd* pParent )
{
    return CDialog::Create(Dlg::IDD, pParent);
}

BOOL Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	char str[50] = "AksPro v";
	strcat(str, vers);

	this->SetWindowText(str);

	if (prefFileSet.iAksType > 4) m_btnOk.EnableWindow(FALSE);
	return TRUE;
}



void Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Radio(pDX, IDC_RADIO1, m_R);
	DDX_Text(pDX, IDC_EDIT1, m_SegQty);
	DDX_Control(pDX, IDC_EDIT1, m_SegQtyWnd);
	DDX_Control(pDX, IDC_PROGRESS1, m_Progress);
	DDX_Check(pDX, IDC_CHECK1, m_SelElems);
	DDX_Check(pDX, IDC_CHECK2, m_Mirror);
	DDX_Control(pDX, IDOK, m_btnOk);
}


BEGIN_MESSAGE_MAP(Dlg, CDialog)
	ON_MESSAGE( WM_ACAD_KEEPFOCUS, onAcadKeepFocus )
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_BUTTON1, OnBnClickedButton1)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN1, OnDeltaposSpin1)
	ON_EN_CHANGE(IDC_EDIT1, OnEnChangeEdit1)
	ON_BN_CLICKED(IDC_RADIO1, OnBnClickedRadio1)
	ON_BN_CLICKED(IDC_RADIO2, OnBnClickedRadio2)
	ON_BN_CLICKED(IDC_RADIO3, OnBnClickedRadio3)
	ON_BN_CLICKED(IDC_RADIO4, OnBnClickedRadio4)
	ON_BN_CLICKED(IDC_RADIO5, OnBnClickedRadio5)
	ON_WM_ACTIVATE()
END_MESSAGE_MAP()

LRESULT Dlg::onAcadKeepFocus( WPARAM, LPARAM )
{
    return TRUE;
}


// Dlg message handlers

void Dlg::OnBnClickedOk()
{


	UpdateData();

	syncCurSetToFileSet();

	prefCurSet.iMirror = m_Mirror;
	prefCurSet.iSelElems = m_SelElems;
	prefCurSet.iAksType = m_R;
	prefCurSet.iSegQty = m_SegQty;

	bDragMode = false;



	if (m_SelElems)
	{
		ShowWindow(SW_HIDE);

		ads_name ss;
		if (RTNORM != acedSSGet( NULL, NULL, NULL, NULL, ss ))
			acutPrintf( "\n \n" );

		acedPostCommandPrompt();

		long length;

		if (!(RTNORM != acedSSLength( ss, &length ) || 0 == length))
			processEntities(ss, length, m_R);

		acedSSFree( ss );
	}
	else
	{
		processDatabase(m_R);
		ShowWindow(SW_HIDE);
	}

	reportNotProcessed();

}


///////////////////////////////////
void Dlg::OnBnClickedButton1()
{
	printVers();
}


void Dlg::OnDeltaposSpin1(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	// TODO: Add your control notification handler code here

	int val = GetDlgItemInt(IDC_EDIT1);

	val+=-pNMUpDown->iDelta;

	if (val < 4) val = 4;

	SetDlgItemInt(IDC_EDIT1, val);

	*pResult = 0;
}

void Dlg::OnEnChangeEdit1()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
	//int val = GetDlgItemInt(IDC_EDIT1);
	//prefCurSet.iSegQty = val;
}

void Dlg::OnBnClickedRadio1()
{
	m_btnOk.EnableWindow();
}

void Dlg::OnBnClickedRadio2()
{
	m_btnOk.EnableWindow();
}

void Dlg::OnBnClickedRadio3()
{
	m_btnOk.EnableWindow();
}

void Dlg::OnBnClickedRadio4()
{
	m_btnOk.EnableWindow();
}

void Dlg::OnBnClickedRadio5()
{
	m_btnOk.EnableWindow();
}


void Dlg::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized)
{
	CDialog::OnActivate(nState, pWndOther, bMinimized);


}

void Dlg::syncToSettings(void)
{
	if (prefFileSet.iAksType < 5) m_R= prefFileSet.iAksType;
	m_SegQty= prefFileSet.iSegQty;
	m_SelElems= prefFileSet.iSelElems;
	m_Mirror= prefFileSet.iMirror;

	//syncCurSetToFileSet();
	//if (prefFileSet.iAksType > 4) m_btnOk.EnableWindow(FALSE);

	UpdateData(FALSE);
}
